@@ -2010,20 +2010,29 @@ protected:
   void TraverseTreeSingleThreadLoopPostorder() {
     _PRAGMA_OMP_SIMD
     for(uint i = 0; i < ParentType::ref_tree_.num_nodes(); i++) {
-      ParentType::ref_spec_.InitNode(i);
+      exception_handler_.Run([=]{
+        ParentType::ref_spec_.InitNode(i);
+      });
     }
+    exception_handler_.Rethrow();
 
     for(uint i = 0; i < ParentType::ref_tree_.num_nodes() - 1; i++) {
-      ParentType::ref_spec_.VisitNode(i);
-      ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
+      exception_handler_.Run([=]{
+        ParentType::ref_spec_.VisitNode(i);
+        ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
+      });
     }
+    exception_handler_.Rethrow();
   }
 
   void TraverseTreeSingleThreadLoopPrunes() {
     _PRAGMA_OMP_SIMD
     for(uint i = 0; i < ParentType::ref_tree_.num_nodes(); i++) {
-      ParentType::ref_spec_.InitNode(i);
+      exception_handler_.Run([=]{
+        ParentType::ref_spec_.InitNode(i);
+      });
     }
+    exception_handler_.Rethrow();
 
     for(uint i_prune = 0;
         i_prune < ParentType::ref_tree_.num_parallel_ranges_prune();
@@ -2032,33 +2041,42 @@ protected:
 
     _PRAGMA_OMP_SIMD
       for(uint i = range_prune[0]; i <= range_prune[1]; i++) {
-        ParentType::ref_spec_.VisitNode(i);
-        ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
+        exception_handler_.Run([=]{
+          ParentType::ref_spec_.VisitNode(i);
+          ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
+        });
       }
+      exception_handler_.Rethrow();
     }
   }
 
   void TraverseTreeSingleThreadLoopVisits() {
     _PRAGMA_OMP_SIMD
     for(uint i = 0; i < ParentType::ref_tree_.num_nodes(); i++) {
-      ParentType::ref_spec_.InitNode(i);
+      exception_handler_.Run([=]{
+        ParentType::ref_spec_.InitNode(i);
+      });
     }
+    exception_handler_.Rethrow();
 
     for(uint i_level = 0; i_level < ParentType::ref_tree_.num_levels(); i_level++) {
       auto range_visit = ParentType::ref_tree_.RangeIdVisitNode(i_level);
       _PRAGMA_OMP_SIMD
       for(uint i = range_visit[0]; i <= range_visit[1]; i++) {
-        if(i < ParentType::ref_tree_.num_tips()) {
-          // i is a tip (only Visit)
-          ParentType::ref_spec_.VisitNode(i);
-        } else {
-          // i is internal
-          for(uint j: ParentType::ref_tree_.FindChildren(i)) {
-            ParentType::ref_spec_.PruneNode(j, i);
+        exception_handler_.Run([=]{
+          if(i < ParentType::ref_tree_.num_tips()) {
+            // i is a tip (only Visit)
+            ParentType::ref_spec_.VisitNode(i);
+          } else {
+            // i is internal
+            for(uint j: ParentType::ref_tree_.FindChildren(i)) {
+              ParentType::ref_spec_.PruneNode(j, i);
+            }
+            ParentType::ref_spec_.VisitNode(i);
           }
-          ParentType::ref_spec_.VisitNode(i);
-        }
+        });
       }
+      exception_handler_.Rethrow();
     }
 
     // VisitNode not called on the root node
@@ -2077,6 +2095,7 @@ protected:
       ParentType::ref_spec_.InitNode(i);  
     });
   }
+  exception_handler_.Rethrow();
 
   uint i_prune = 0;
   for(uint i_level = 0; i_level < ParentType::ref_tree_.num_levels(); i_level++) {
@@ -2090,6 +2109,7 @@ protected:
           ParentType::ref_spec_.VisitNode(i);
         });
       }
+      exception_handler_.Rethrow();
 
       uint num_branches_done = 0;
 
@@ -2103,6 +2123,7 @@ protected:
             ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
           });
         }
+        exception_handler_.Rethrow();
 
         num_branches_done +=  range_prune[1] - range_prune[0] + 1;
       ++i_prune;
@@ -2127,6 +2148,7 @@ protected:
         ParentType::ref_spec_.InitNode(i);
       });
     }
+    exception_handler_.Rethrow();
 
     for(uint i_level = 0; i_level < ParentType::ref_tree_.num_levels(); i_level++) {
       auto range_visit = ParentType::ref_tree_.RangeIdVisitNode(i_level);
@@ -2145,6 +2167,7 @@ protected:
           }
         });
       }
+      exception_handler_.Rethrow();
     }
 }
     // VisitNode not called on the root node
@@ -2190,6 +2213,7 @@ protected:
     }
   });
 }
+exception_handler_.Rethrow();
 }
 
   void TraverseTreeMultiThreadLoopPrunes() {
@@ -2202,6 +2226,7 @@ protected:
       ParentType::ref_spec_.InitNode(i);
     });
   }
+  exception_handler_.Rethrow();
 
   for(uint i_prune = 0; i_prune < ParentType::ref_tree_.num_parallel_ranges_prune(); i_prune++) {
     auto range_prune = ParentType::ref_tree_.RangeIdPruneNode(i_prune);
@@ -2213,6 +2238,7 @@ protected:
           ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
         });
       }
+      exception_handler_.Rethrow();
   }
 }
   }
@@ -2255,6 +2281,7 @@ protected:
         ParentType::ref_spec_.InitNode(i);
       });
     }
+    exception_handler_.Rethrow();
 
     uint i_prune = 0;
   for(uint i_level = 0; i_level < ParentType::ref_tree_.num_levels(); i_level++) {
@@ -2268,6 +2295,7 @@ protected:
             ParentType::ref_spec_.VisitNode(i);
           });
         }
+        exception_handler_.Rethrow();
     } else if(tid == 0) {
       // only the master thread executes this
       _PRAGMA_OMP_SIMD
@@ -2276,6 +2304,7 @@ protected:
           ParentType::ref_spec_.VisitNode(i);
         });
       }
+      exception_handler_.Rethrow();
     }
 
     if (tid == 0) {
@@ -2289,6 +2318,7 @@ protected:
               ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
             });
           }
+          exception_handler_.Rethrow();
 
           num_branches_done +=  range_prune[1] - range_prune[0] + 1;
         ++i_prune;
@@ -2315,6 +2345,7 @@ protected:
         ParentType::ref_spec_.InitNode(i);
       });
     }
+    exception_handler_.Rethrow();
 
 
   for(uint i_prune = 0; i_prune < ParentType::ref_tree_.num_parallel_ranges_prune(); i_prune++) {
@@ -2329,6 +2360,7 @@ protected:
             ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
           });
         }
+        exception_handler_.Rethrow();
       } else if (tid == 0) {
         // only one (master) thread executes this
         _PRAGMA_OMP_SIMD
@@ -2338,6 +2370,7 @@ protected:
             ParentType::ref_spec_.PruneNode(i, ParentType::ref_tree_.FindIdOfParent(i));
           });
         }
+        exception_handler_.Rethrow();
       }
     }
 }
@@ -2360,6 +2393,7 @@ protected:
         ParentType::ref_spec_.InitNode(i);
       });
     }
+    exception_handler_.Rethrow();
 
   for(uint i_level = 0; i_level < ParentType::ref_tree_.num_levels(); i_level++) {
     auto range_visit = ParentType::ref_tree_.RangeIdVisitNode(i_level);
@@ -2381,6 +2415,7 @@ protected:
           }
         });
       }
+      exception_handler_.Rethrow();
     } else if(tid == 0) {
       // only the master thread executes this
       _PRAGMA_OMP_SIMD
@@ -2398,6 +2433,7 @@ protected:
           }
         });
       }
+      exception_handler_.Rethrow();
     }
   }
 }
@@ -2407,6 +2443,7 @@ protected:
         ParentType::ref_spec_.PruneNode(j, ParentType::ref_tree_.num_nodes() - 1);
       });
     }
+    exception_handler_.Rethrow();
   }
 };
 
